==> euler-math-0.4.0.tar.gz: euler-math-0.4.0/src/euler_math/tco.py <==
# tail recursion optimization
# from http://metapython.blogspot.com.br/2010/11/tail-recursion-elimination-in-python.html

from threading import currentThread


class TailRecursiveCall(Exception):
    pass


class Rec_f(object):

    def __init__(self, func):
        self.tr_d = {}
        self.func = func

    def __call__(self, *args, **kw):
        self.args = args
        self.kw = kw
        thread = currentThread()
        if thread not in self.tr_d:
            self.tr_d[thread] = {}
            self.tr_d[thread]["depth"] = 0
        self.tr_d[thread]["depth"] += 1
        # Record the arguments passed to function on this thread
        self.tr_d[thread]["args"] = args
        self.tr_d[thread]["kw"] = kw
        depth = self.tr_d[thread]["depth"]
        # If we are re-entering the same function on the same thread:
        if depth > 1:
            # Effectively detours execution to the "Landing Point",
            # but two execution frames above the current one:
            raise TailRecursiveCall
        over = False
        while not over:
            over = True
            args = self.tr_d[thread]["args"]
            kw = self.tr_d[thread]["kw"]
            try:
                # Execute the function with the latest arguments for this thread.
                result = self.func(*args, **kw)
            except TailRecursiveCall:
                # Landing point if the function tried to recurse itself.
                # Two execution frames above.
                self.tr_d[thread]["depth"] -= 1
                over = False
        self.tr_d[thread]["depth"] -= 1
        return result


def tco(func):
    return Rec_f(func)

==> euler-math-0.4.5.tar.gz: euler-math-0.4.5/src/euler_math/tco.py <==
"""
tail recursion optimization
from http://metapython.blogspot.com.br/2010/11/tail-recursion-elimination-in-python.html
"""
from threading import currentThread


class TailRecursiveCall(Exception):
    """
    error raised if function tries to recurse itself
    """

class RecursiveFunction:
    """
    Main Tail Recursive Class
    """
    def __init__(self, func):
        self.tr_d = {}
        self.func = func
        self.args = None
        self.kw = None

    def __call__(self, *args, **kw):
        self.args = args
        self.kw = kw
        thread = currentThread()
        if thread not in self.tr_d:
            self.tr_d[thread] = {}
            self.tr_d[thread]["depth"] = 0
        self.tr_d[thread]["depth"] += 1
        # Record the arguments passed to function on this thread
        self.tr_d[thread]["args"] = args
        self.tr_d[thread]["kw"] = kw
        depth = self.tr_d[thread]["depth"]
        # If we are re-entering the same function on the same thread:
        if depth > 1:
            # Effectively detours execution to the "Landing Point",
            # but two execution frames above the current one:
            raise TailRecursiveCall
        over = False
        while not over:
            over = True
            args = self.tr_d[thread]["args"]
            kw = self.tr_d[thread]["kw"]
            try:
                # Execute the function with the latest arguments for this thread.
                result = self.func(*args, **kw)
            except TailRecursiveCall:
                # Landing point if the function tried to recurse itself.
                # Two execution frames above.
                self.tr_d[thread]["depth"] -= 1
                over = False
        self.tr_d[thread]["depth"] -= 1
        return result


def tco(func):
    """
    Function decorator for tail recursion optimization.
    """
    return RecursiveFunction(func)
